///////////////////////////////////////////////////////////////////////////////
// File			tree_node_2.hpp
// Version		0.9
// Copyright	Orjan Westin 2006
// Licence		BSD (see EOF comment)
// Contact		orjan.westin@cunobaros.com
// Download		http://www.cunobaros.com/software/trees_roots_leaves2_source.zip
///////////////////////////////////////////////////////////////////////////////

#include <functional>
#include <numeric>

namespace cunobaros
{
template <typename T >
class tree_node_2
{
public:
    typedef tree_node_2<T> node_type;
    typedef tree_node_2<T>& reference;
	typedef const tree_node_2<T>& const_reference;
    typedef tree_node_2<T>* pointer;
	typedef const tree_node_2<T>* const_pointer;
    typedef T& value_reference;
    typedef const T& const_value_reference;
    typedef T* value_pointer;
	typedef const T* const_value_pointer;
	typedef std::vector<pointer> child_holder;
	typedef typename child_holder::size_type size_type;
	typedef typename child_holder::iterator iterator;
	typedef typename child_holder::const_iterator const_iterator;

    tree_node_2():
		value_(NULL),
        parent_(NULL){;}

    tree_node_2( const_value_reference value ):
		value_( new T( value ) ),
        parent_(NULL){;}

    ~tree_node_2()
    {
		delete value_;
        clear();
	}

    bool has_value() const
    {
		return (NULL != value_ );
	}

    value_reference value()
    {
		if ( !has_value() )
            throw std::exception("No value");
        return *value_;
    }

    const_value_reference value() const
    {
		if ( !has_value() )
            throw std::exception("No value");
        return *value_;
    }

    void assign_value(const_value_reference value)
    {
		if ( has_value())
            delete value_;
        value_= new T(value);
    }

    pointer parent()
    {
		return parent_;
	}

    pointer root()
    {
		pointer tmp = this;
        while ( NULL == tmp->parent() )
            tmp = tmp->parent();
        return tmp;
    }

	iterator begin()
  	{
		return children_.begin();
	}
	iterator end()
  	{
		return children_.end();
	}
	const_iterator begin() const
  	{
		return children_.begin();
	}
	const_iterator end() const
  	{
		return children_.end();
	}

  	reference front()
	{
		return *children_.front();
	}
  	reference back()
	{
		return *children_.back();
	}
  	const_reference front() const
	{
		return *children_.front();
	}
  	const_reference back() const
	{
		return *children_.back();
	}

	void adopt(const_value_reference child)
	{
		adopt(create_value_node(&child));
	}
	iterator adopt(pointer child)
    {
		if (NULL == child)
			throw std::invalid_argument("Cannot adopt NULL pointers");
        iterator i = find_child(child);
		if (i == children_.end())
        {
    		children_.push_back(child);
            i = --children_.end();
        }
		child->reparent(this);
        return i;
	}

	pointer elope()
	{
	    if (NULL != parent_)
        {
			parent_->eloping(this);
			parent_ = NULL;
		}
		return this;
	}

	pointer disown(iterator child)
	{
		if ((child != children_.end()) && (this == (*child)->parent()))
			return (*child)->elope();
		throw std::invalid_argument("Cannot find child to disown");
	}

	iterator erase(iterator it)
	{
		pointer p = *it;
		iterator next = children_.erase(it);
		p->erase_silent();
		return next;
	}
	iterator erase(iterator first, iterator last)
	{
		child_holder backup(first, last);
		iterator next = children_.erase(first, last);
		std::for_each(backup.begin(), backup.end(), std::mem_fun(&node_type::erase_silent));
		return next;
	}
    void clear()
    {
        erase(begin(), end());
    }

	void copy_contents(const_reference node)
	{
	    node_type temp;
	    temp.copy_children(node);
        if (node.has_value())
            temp.assign_value(node.value());
        // Make room if needed
	    if (temp.size() > children_.capacity())
		    children_.reserve(temp.size());
        clear();
        while (temp.size())
            adopt(*temp.begin());
	    value_ = temp.value_;
	    temp.value_ = NULL;
	}
	bool empty() const
    {
        return children_empty();
    }
	size_type size() const
    {
        return children_.size();
    }
	size_type total_size() const
    {
	    size_type result(size());
	    result = std::accumulate<const_iterator, size_type>(begin(), end(), result, AccumulateSize());
	    return result;
    }
private:
	void copy_children(const_reference node)
	{
		clear();
		std::for_each(node.begin(), node.end(), std::bind1st( AdoptCopy(), this));
	}

	static pointer create_value_node(const_value_pointer child)
	{
		if (NULL != child)
			return new node_type(*child);
		return new node_type();
	}

	static pointer create_copy(const_pointer pnode)
	{
		pointer p = NULL;
		try
        {
			p = create_value_node(pnode->value_);
			std::for_each(pnode->begin(), pnode->end(), std::bind1st(AdoptCopy(), p));
		}
		catch (...)
		{
			delete p;
			throw;
		}
		return p;
	}

	bool erase_silent()
	{
		parent_ = NULL;
		delete this;
		return true;
	}

	void reparent(pointer parent)
	{
		if (parent == parent_)
			return;
		elope();
		parent_ = parent;
	}

	void eloping(pointer child)
	{
		iterator ch = find_child(child);
		if (ch != children_.end())
			children_.erase(ch);
	}

	iterator find_child(const pointer child)
	{
		return std::find(children_.begin(), children_.end(), child);
	}

	// Helper functors
	struct AdoptCopy : public std::binary_function<pointer, const_pointer, bool>
	{
		bool operator()(pointer lhs, const_pointer rhs) const
		{
			lhs->adopt(create_copy(rhs));
			return true;
		}
	};

	struct AccumulateSize
    {
		size_type operator()(size_type val, pointer const p) const
        {
			val += p->size();
			val = std::accumulate<const_iterator, size_type>(p->begin(), p->end(), val, AccumulateSize());
			return val;
        }
    };

    pointer parent_;
    T* value_;
	child_holder children_;
};

}	// namespace cunobaros
/*

Copyright (c) 2006, Orjan Westin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

The name of the contributor(s) may not be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/
