// tree_test.cpp : Defines the entry point for the console application.
//

#include <string>
#include <vector>
#include <fstream>
#include "tree_node_2.hpp"
#include <functional>
#include <algorithm>

// Simple XML element
class xml_element
{
	std::string el;
public:
	xml_element(){;}
	xml_element(const std::string & name):el(name){;}
	xml_element(const xml_element & lmt):el(lmt.el){;}
	// ...
	// These functions prints the opening and closing tags
	std::ostream & print_opening(std::ostream & ostr) const
		{ostr << '<' << el << '>'; return ostr; }
	std::ostream & print_closing(std::ostream & ostr) const
		{ostr << "</" << el << '>'; return ostr; }
};

// A tree of XML elements is an XML document
typedef cunobaros::tree_node_2<xml_element> xml_doc;

// A recursive functor to stream out an xml_doc
struct xml_doc_stream_out : public std::binary_function<xml_doc::const_pointer, std::ostream *, std::ostream &>
{
	std::ostream & operator()(xml_doc::const_pointer p, std::ostream * pstr) const
	{
		p->value().print_opening(*pstr);
		std::for_each(p->begin(), p->end(), std::bind2nd(xml_doc_stream_out(), pstr));
		p->value().print_closing(*pstr);
		return *pstr;
	}
};

// The usual two output streams
std::ostream & operator<<(xml_doc::const_reference node, std::ostream & ostr)
{
	xml_doc_stream_out()(&node, &ostr);
	return ostr;
}

std::ostream & operator<<(std::ostream & ostr, xml_doc::const_reference node)
{
	xml_doc_stream_out()(&node, &ostr);
	return ostr;
}

int main(int argc, char* argv[])
{
	std::ofstream s;
	s.open("result.txt");

	// Using implicit creation of xml_element through assignment constructor
	std::string root_name("mama");
	xml_doc root(root_name);
	// And adoption
	std::string child1_name("child_1");
	root.adopt(child1_name);
	// Explicit works just as fine
	root.adopt(xml_element("child_2"));

	// Get a step down the tree
	root.front().adopt(xml_element("grandchild"));

	// Print to see what we have
	s << "Initial layout -\n ";
	s << root << "\n ";
	s << "Total number of children under root : " << root.total_size() << "\n ";
	s << "Children of root: " << root.size() << "\n";

	// Take a look at first child
	s << "Step down -\n ";
	s << root.front() << "\n";

	// Make a copy before modifying root
	xml_doc copy;
	copy.copy_contents(root);
	copy.value() = xml_element("papa");

	// Disown first child
	xml_doc::pointer p = root.disown(root.begin());
	// Since it's disowned, we are now responsible for its future
	// We'll take care of its child ...
	xml_doc::pointer p2 = p->disown(p->begin());
	// ... by giving it another parent...
	root.front().adopt(p2);
	// ... and even give it a child of its own
	p2->adopt(xml_element("baby"));
	// Don't forget to get rid of the one we have no more use for
	delete p;
	// Remember NOT to delete p2, as it is now adopted and will be deleted by its parent
	// But it's simple to check if we're uncertain
	if (NULL == p2->parent())
		delete p2;

	// Look at changed relationships
	s << "Reshuffle -\n ";
	s << root << "\n ";
	s << "Total number of children under root : " << root.total_size() << "\n ";
	s << "Children of root: " << root.size() << "\n";
	s << "Copy of original -\n ";
	s << copy << "\n ";

	s.close();
	return 0;
}
